# Generated by roxygen2: do not edit by hand

S3method(compare_category,data.frame)
S3method(compare_numeric,data.frame)
S3method(correlate,data.frame)
S3method(correlate,grouped_df)
S3method(correlate,tbl_dbi)
S3method(describe,data.frame)
S3method(describe,grouped_df)
S3method(describe,tbl_dbi)
S3method(diagnose,data.frame)
S3method(diagnose,grouped_df)
S3method(diagnose,tbl_dbi)
S3method(diagnose_category,data.frame)
S3method(diagnose_category,grouped_df)
S3method(diagnose_category,tbl_dbi)
S3method(diagnose_numeric,data.frame)
S3method(diagnose_numeric,grouped_df)
S3method(diagnose_numeric,tbl_dbi)
S3method(diagnose_outlier,data.frame)
S3method(diagnose_outlier,grouped_df)
S3method(diagnose_outlier,tbl_dbi)
S3method(diagnose_paged_report,data.frame)
S3method(diagnose_paged_report,tbl_dbi)
S3method(diagnose_report,data.frame)
S3method(diagnose_report,tbl_dbi)
S3method(diagnose_sparese,data.frame)
S3method(diagnose_web_report,data.frame)
S3method(diagnose_web_report,tbl_dbi)
S3method(eda_paged_report,data.frame)
S3method(eda_paged_report,tbl_dbi)
S3method(eda_report,data.frame)
S3method(eda_report,tbl_dbi)
S3method(eda_web_report,data.frame)
S3method(eda_web_report,tbl_dbi)
S3method(extract,bins)
S3method(imputate_na,data.frame)
S3method(imputate_outlier,data.frame)
S3method(normality,data.frame)
S3method(normality,grouped_df)
S3method(normality,tbl_dbi)
S3method(plot,bins)
S3method(plot,compare_category)
S3method(plot,compare_numeric)
S3method(plot,correlate)
S3method(plot,imputation)
S3method(plot,infogain_bins)
S3method(plot,optimal_bins)
S3method(plot,overview)
S3method(plot,performance_bin)
S3method(plot,pps)
S3method(plot,relate)
S3method(plot,transform)
S3method(plot,univar_category)
S3method(plot,univar_numeric)
S3method(plot_bar_category,data.frame)
S3method(plot_bar_category,grouped_df)
S3method(plot_box_numeric,data.frame)
S3method(plot_box_numeric,grouped_df)
S3method(plot_correlate,data.frame)
S3method(plot_correlate,grouped_df)
S3method(plot_correlate,tbl_dbi)
S3method(plot_hist_numeric,data.frame)
S3method(plot_hist_numeric,grouped_df)
S3method(plot_normality,data.frame)
S3method(plot_normality,grouped_df)
S3method(plot_normality,tbl_dbi)
S3method(plot_outlier,data.frame)
S3method(plot_outlier,target_df)
S3method(plot_outlier,tbl_dbi)
S3method(plot_qq_numeric,data.frame)
S3method(plot_qq_numeric,grouped_df)
S3method(pps,data.frame)
S3method(pps,target_df)
S3method(print,bins)
S3method(print,compare_category)
S3method(print,compare_numeric)
S3method(print,relate)
S3method(print,transform)
S3method(print,univar_category)
S3method(print,univar_numeric)
S3method(relate,target_df)
S3method(summary,bins)
S3method(summary,compare_category)
S3method(summary,compare_numeric)
S3method(summary,correlate)
S3method(summary,imputation)
S3method(summary,optimal_bins)
S3method(summary,overview)
S3method(summary,performance_bin)
S3method(summary,pps)
S3method(summary,transform)
S3method(summary,univar_category)
S3method(summary,univar_numeric)
S3method(target_by,data.frame)
S3method(target_by,tbl_dbi)
S3method(univar_category,data.frame)
S3method(univar_numeric,data.frame)
export(binning)
export(binning_by)
export(binning_rgr)
export(compare_category)
export(compare_numeric)
export(correlate)
export(cramer)
export(describe)
export(diagnose)
export(diagnose_category)
export(diagnose_numeric)
export(diagnose_outlier)
export(diagnose_paged_report)
export(diagnose_report)
export(diagnose_sparese)
export(diagnose_web_report)
export(dlookr_blue_paged)
export(dlookr_orange_paged)
export(dlookr_templ_html)
export(eda_paged_report)
export(eda_report)
export(eda_web_report)
export(entropy)
export(extract)
export(find_class)
export(find_na)
export(find_outliers)
export(find_skewness)
export(get_class)
export(get_column_info)
export(get_os)
export(get_percentile)
export(get_transform)
export(import_google_font)
export(imputate_na)
export(imputate_outlier)
export(jsd)
export(kld)
export(kurtosis)
export(normality)
export(overview)
export(performance_bin)
export(plot_bar_category)
export(plot_box_numeric)
export(plot_correlate)
export(plot_hist_numeric)
export(plot_na_hclust)
export(plot_na_intersect)
export(plot_na_pareto)
export(plot_normality)
export(plot_outlier)
export(plot_qq_numeric)
export(pps)
export(relate)
export(skewness)
export(target_by)
export(theil)
export(transform)
export(transformation_paged_report)
export(transformation_report)
export(transformation_web_report)
export(univar_category)
export(univar_numeric)
import(dplyr)
import(ggplot2)
import(hrbrthemes)
import(htmltools)
import(pagedown)
import(reactable)
import(showtext)
import(tibble)
importFrom(dplyr,rename)
importFrom(grDevices,boxplot.stats)
importFrom(grDevices,cairo_pdf)
importFrom(grDevices,col2rgb)
importFrom(grDevices,nclass.Sturges)
importFrom(grDevices,rgb)
importFrom(graphics,boxplot)
importFrom(graphics,hist)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(grid,gpar)
importFrom(grid,textGrob)
importFrom(gridExtra,arrangeGrob)
importFrom(gridExtra,grid.arrange)
importFrom(hrbrthemes,scale_fill_ipsum)
importFrom(hrbrthemes,theme_ipsum)
importFrom(hrbrthemes,theme_ipsum_rc)
importFrom(htmltools,HTML)
importFrom(htmltools,br)
importFrom(htmltools,div)
importFrom(htmltools,h3)
importFrom(htmltools,h4)
importFrom(htmltools,plotTag)
importFrom(kableExtra,add_header_above)
importFrom(kableExtra,kable_styling)
importFrom(knitr,image_uri)
importFrom(knitr,kable)
importFrom(knitr,knit2pdf)
importFrom(methods,is)
importFrom(pagedown,chrome_print)
importFrom(pagedown,find_chrome)
importFrom(purrr,map)
importFrom(purrr,map_dbl)
importFrom(purrr,map_df)
importFrom(purrr,map_dfc)
importFrom(purrr,map_dfr)
importFrom(purrr,map_int)
importFrom(purrr,map_lgl)
importFrom(purrr,walk)
importFrom(rlang,enquo)
importFrom(rlang,quos)
importFrom(rlang,set_names)
importFrom(rlang,warn)
importFrom(rmarkdown,render)
importFrom(shiny,icon)
importFrom(shiny,tabPanel)
importFrom(shiny,tabsetPanel)
importFrom(stats,IQR)
importFrom(stats,addmargins)
importFrom(stats,aggregate)
importFrom(stats,anova)
importFrom(stats,approx)
importFrom(stats,as.dendrogram)
importFrom(stats,chisq.test)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,dist)
importFrom(stats,formula)
importFrom(stats,hclust)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,order.dendrogram)
importFrom(stats,predict)
importFrom(stats,qqline)
importFrom(stats,qqnorm)
importFrom(stats,quantile)
importFrom(stats,reorder)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,shapiro.test)
importFrom(stats,var)
importFrom(stats,xtabs)
importFrom(sysfonts,font_add_google)
importFrom(sysfonts,font_families)
importFrom(tibble,as_tibble)
importFrom(tibble,column_to_rownames)
importFrom(tibble,enframe)
importFrom(tibble,is_tibble)
importFrom(tibble,rownames_to_column)
importFrom(tidyr,gather)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,spread)
importFrom(tidyr,unnest)
importFrom(tidyselect,all_of)
importFrom(tidyselect,matches)
importFrom(tidyselect,vars_select)
importFrom(utils,browseURL)
importFrom(utils,combn)
importFrom(utils,getFromNamespace)
importFrom(utils,head)
importFrom(utils,packageVersion)
